﻿from remoteapi import *
import PIDataReferenceFrame_pb2

"""
TestAddToPlatform.py

Unit test for the Remote API using Python that validates the usage of supplied ID
values across multiple connections.

"""

def createClient(desc):
	"""
	Create a default client connection to SIMDIS and initialize default settings.
	Raises an exception on failure to connect.
	"""
	dc = remoteapi.DiscoveryClient()
	dc.description = desc
	client = dc.createClient()
	if client is None:
		raise Exception("Failed to connect.")
	client.pluginApi().config().setResponsesEnabled(True)
	return client

def newAddPlatformPointCommand(id, time, latLonAlt):
	""" Returns a RemoteApi_pb2.Command with the platform point specified. """
	command = RemoteApi_pb2.Command()
	addData = command.piData.platform.addData
	addData.id = id
	addData.frame.system = PIDataReferenceFrame_pb2.COORDSYS_LLA
	p = addData.points.add()
	p.time = time
	p.position.x = latLonAlt[0]
	p.position.y = latLonAlt[1]
	p.position.z = latLonAlt[2]
	return command

def addPlatform(client, localId, xoffset):
	"""
	Creates a new platform using the supplied localId on the given client.
	Will auto-configure clock at the end and wait for responses to finish.
	"""
	# Create a platform with a data point at time 0
	command = RemoteApi_pb2.Command()
	create = command.piData.platform.create
	create.header.name = 'Python Platform'
	create.header.originalId = localId
	create.header.iconName = 'x'
	create.suppliedId = localId
	create.frame.system = PIDataReferenceFrame_pb2.COORDSYS_LLA
	p = create.points.add()
	p.time = p.position.y = p.position.z = 0.0
	p.position.x = 0.1 + xoffset
	client.send(command)

	# We should be able to add at time 10.0
	command = newAddPlatformPointCommand(localId, 10.0, [0.2 + xoffset, 0.1, 0])
	client.send(command)

	# Fix the time and wait for responses
	client.pluginApi().data().clockAutoConfigureBeginEnd().waitForResponse(1000)

def addPlatformNewConnection():
	""" Creates a new connection, creates a platform, and closes. """
	client = createClient('TATP.addPlatformNewConnection')
	addPlatform(client, 1234, 0)
	client.close()

def init():
	"""
	Deletes any existing scenario and adds a single platform using a
	new client connection.
	"""
	client = createClient('TATP.init')
	pi = client.pluginApi()

	# Create a new scenario, clearing out the old one
	pi.data().deleteScenario()
	pi.data().initializeScenario("Test Add To Platform", 0, 0, 0)
	pi.data().setOriginLLA(0.1, 0.2, 1000)
	pi.data().setReferenceYear(2016)
	pi.data().setDescription("Tests entity creation with multiple clients")
	pi.data().setScenarioFilename("TestAddToPlatform.py.asi")
	# RGBA = 0xRRGGBBAA
	pi.data().setClassification('UNCLASSIFIED', 0x00ff0080)

	addPlatform(client, 1234, 0)
	client.close()
	print("Scenario initialized.")

def testAddByFindId():
	"""
	Attempts to add data to a platform in a new client, using find-platform method
	of finding the platform created by another client.
	"""
	client = createClient('TATP.testAddByFindId')
	pi = client.pluginApi()
	# Must first find platform ID; returns only the first one found
	rv = pi.data().findPlatform("Python Platform")
	if not rv.isOk(1000):
		raise Exception("Failed to find platform 'Python Platform'.  Did you run initialization script?")

	# We should be able to add at time 22.0 because we are using the ID from findPlatform()
	command = newAddPlatformPointCommand(rv.value(), 22.0, [0.225, 0.11, 0])
	rv = PluginApi.ResponseHandler(client, client.send(command), None)

	# Make sure the add DID succeed
	if not rv.isOk(1000):
		raise Exception("Unexpected failure  from creating platform.")
	print("Successfully added data point to platform.  Test passed.")

	# Fix the time and wait for responses
	pi.data().clockAutoConfigureBeginEnd().waitForResponse(1000)
	client.close()

def testAddWithoutFind_ExpectedFail():
	"""
	Attempts to add data to a platform in a new client, without using the find-
	platform method.  Instead, the ID is specifically provided (1234) with an
	erroneous expectation that the IDs from this new client will match the client
	from the init() call.  This won't work due to ID separation.  So we test to
	ensure that this fails successfully.
	"""
	client = createClient('TATP.testAddWithoutFind_ExpectedFail')
	pi = client.pluginApi()
	# We should NOT be able to add because ID 1234 will be invalid to this Remote API connection
	command = newAddPlatformPointCommand(1234, 17.0, [0.25, 0.1, 0])
	rv = PluginApi.ResponseHandler(client, client.send(command), None)

	# Make sure the add did NOT succeed
	if rv.isOk(1000):
		raise Exception("Unexpected success from creating platform.")

	# Fix the time and wait for responses
	pi.data().clockAutoConfigureBeginEnd().waitForResponse(1000)
	client.close()
	print("Successfully failed to add data point to unknown platform.  Test passed.")

def testAnimatedLine():
	client = createClient('TATP.testAddByFindId')
	pi = client.pluginApi()
	# Must first find platform ID; returns only the first one found
	rv = pi.data().findPlatform("Python Platform")
	if not rv.isOk(1000):
		raise Exception("Failed to find platform 'Python Platform'.  Did you run initialization script?")
	platformId = rv.value()

	addPlatform(client, 47, 0.01)

	rv = pi.data().addGenericData(platformId, 0, "SIMDIS_AnimatedLine 1", "entityid=47,stipple=0xFF00:0x00FF,color=0xFF14FF0E:0xFFFF4700,width=3.0,shiftsPerSecond=60.0,numSegments=1");
	if not rv.isOk(1000):
		raise Exception("Failed to add generic data")

	client.close()
	print("Successfully added generic data.  Test passed.")

init()
# Make two dummy platforms to attempt to confuse the test
addPlatformNewConnection()
addPlatformNewConnection()
testAddByFindId()
testAddWithoutFind_ExpectedFail()
testAnimatedLine()
