import io
import math
import RemoteApi_pb2
import remoteapi
import remoteapi.PluginApi
import ScenarioHelper
import sys
import time
import zlib

"""
TestFastLargeVideo.py

Unit test for Remote API that displays several high-resolution images, and demonstrates Remote API performance under stress.
"""

# Sentinel for deciding to generate images or use command line arguments
COMMAND_LINE = False
# Sentinels for erroneous exit codes
MISSING_PIL = 1
FILE_MISSING = 2
# Size of images to be generated and displayed.
IMAGE_WIDTH = 2048
IMAGE_HEIGHT = 1024
# Target framerate to be tested/achieved.
TARGET_FRAMERATE = 30

try:
	from PIL import Image
except ImportError:
	print('This script requires Pillow (PIL) to be installed (can run "pip install Pillow" in a shell that has python in its path to obtain).')
	sys.exit(MISSING_PIL)

# Gets bytes from a binary stream and passes through remoteAPI function call to update texture.
# This is a performance test, so checking for responses to the api call is unnecessary.
def updateImage(binaryStreamBytes, textureId):
	f = piSimdis.updateTextureStream(textureId, binaryStreamBytes, 0, 0)

# Attempt to quickly pass images through remoteAPI for 5 seconds at specified framerate.
def startStream(imgOneBytes, imgTwoBytes, textureId):
	usefirstImage = True
	for x in range(5):
		count = 0
		timeEnd = time.time() + 1
		while time.time() < timeEnd and count < TARGET_FRAMERATE:
			streamBytes = imgOneBytes if usefirstImage else imgTwoBytes
			updateImage(streamBytes, textureId)
			usefirstImage = not usefirstImage
			count += 1
		if time.time() < timeEnd:
			time.sleep(timeEnd - time.time()) 
		print(f"Made {count} api calls this second")

# Creates images and stores compressed imagery data (i.e. png format images) into an in-memory binary stream.
# This stream can be used by the different remoteAPI texture calls in PISIMDISSender.py.
# Image is currently size 2048x1024 with 4x8-bit pixels ("true color with transparency mask").
# SIMDIS converts images to resolutions that are powers of 2, so performs better when images can be used without conversion.
def initImageBinaryStream(color):
	img = Image.new('RGBA', (IMAGE_WIDTH, IMAGE_HEIGHT), color)
	imgBinaryStream = io.BytesIO()
	img.save(imgBinaryStream, "png")
	return imgBinaryStream

# Loads an image from a file location to be used for "video".
def initImageBinaryStreamFromLoad(imageName):
	try:
		img = Image.open(f"{imageName}.png")
		imgBinaryStream = io.BytesIO()
		img.save(imgBinaryStream, "png")
		return imgBinaryStream
	except FileNotFoundError:
		print(f"File {imageName}.png was missing.")
		exit(FILE_MISSING)

client = remoteapi.DiscoveryClient().createClient()
try:
	if client is None:
		raise Exception("Failed to connect.")

	# Checks for command line arguments (file names).
	commandLineArgsLen = len(sys.argv)
	if commandLineArgsLen == 3:
		COMMAND_LINE = True
	elif commandLineArgsLen > 1 and commandLineArgsLen != 3:
		print("Need two png filenames as command line arguments to use images from disk.")
		print("Running as if none were provided.")

	pi = client.pluginApi()
	# Performance test, so checking for responses is unnecessary.
	pi.config().setResponsesEnabled(False)
	piData = pi.data()
	piSimdis = pi.simdis()

	# Get image data in the form of bytes to pass through remoteAPI calls.
	if COMMAND_LINE:
		imgOneStreamBytes = initImageBinaryStreamFromLoad(sys.argv[1]).getvalue()
		imgTwoStreamBytes = initImageBinaryStreamFromLoad(sys.argv[2]).getvalue()
	else:
		imgOneStreamBytes = initImageBinaryStream('red').getvalue()
		imgTwoStreamBytes = initImageBinaryStream('blue').getvalue()

	# New platform
	ScenarioHelper.newScenario(piData, origin=[math.radians(22), math.radians(-159),0])
	f = ScenarioHelper.createPlatform(client, 20, [math.radians(22.1), math.radians(-159), 0.0]) # slightly north
	platId = 20

	# Set a reasonable view
	piSimdis.tetherCamera("MainView", platId)
	piSimdis.setCameraRAE("MainView", 2500.0, 0.0, 0.3)

	# Create a texture
	f = piSimdis.addTextureStream(imgOneStreamBytes, 0, "Python Image", 100)
	textureId = 100

	# Assign the texture to the platform
	f = piSimdis.setPlatformTextureIcon(platId, textureId)

	# Start passing images to SIMDIS from remote api
	startStream(imgOneStreamBytes, imgTwoStreamBytes, textureId)

	# Clean up the texture memory
	f = piSimdis.removeTexture(textureId)

finally:
	client.close()
