import math
import RemoteApi_pb2
import remoteapi
import remoteapi.PluginApi
import ScenarioHelper
import time
import zlib
import struct

"""
TestProjectorTexture.py

Unit test for Remote API Projector Texture changing
"""

# Adapted from https://stackoverflow.com/questions/8554282
def makeGrayPNG(data, height = None, width = None):
	def I1(value):
		return struct.pack("!B", value & (2**8-1))
	def I4(value):
		return struct.pack("!I", value & (2**32-1))
	# compute width&height from data if not explicit
	if height is None:
		height = len(data) # rows
	if width is None:
		width = 0
		for row in data:
			if width < len(row):
				width = len(row)
	# generate these chunks depending on image type
	makeIHDR = True
	makeIDAT = True
	makeIEND = True
	png = b"\x89" + "PNG\r\n\x1A\n".encode('ascii')
	if makeIHDR:
		colortype = 0 # true gray image (no palette)
		bitdepth = 8 # with one byte per pixel (0..255)
		compression = 0 # zlib (no choice here)
		filtertype = 0 # adaptive (each scanline seperately)
		interlaced = 0 # no
		IHDR = I4(width) + I4(height) + I1(bitdepth)
		IHDR += I1(colortype) + I1(compression)
		IHDR += I1(filtertype) + I1(interlaced)
		block = "IHDR".encode('ascii') + IHDR
		png += I4(len(IHDR)) + block + I4(zlib.crc32(block))
	if makeIDAT:
		raw = b""
		for y in range(height):
			raw += b"\0" # no filter for this scanline
			for x in range(width):
				c = b"\0" # default black pixel
				if y < len(data) and x < len(data[y]):
					c = I1(data[y][x])
				raw += c
		compressor = zlib.compressobj()
		compressed = compressor.compress(raw)
		compressed += compressor.flush() #!!
		block = "IDAT".encode('ascii') + compressed
		png += I4(len(compressed)) + block + I4(zlib.crc32(block))
	if makeIEND:
		block = "IEND".encode('ascii')
		png += I4(0) + block + I4(zlib.crc32(block))
	return png

# Make 4x4 icons to avoid rescaling
crossPng = makeGrayPNG([[0,255,255,0],[255,255,255,255],[255,255,255,255],[0,255,255,0]])
xPng = makeGrayPNG([[255,0,0,255],[0,255,255,0],[0,255,255,0],[255,0,0,255]])

client = remoteapi.DiscoveryClient().createClient()
try:
	if client is None:
		raise Exception("Failed to connect.")

	pi = client.pluginApi()
	piData = pi.data()
	piSimdis = pi.simdis()

	ScenarioHelper.newScenario(piData, origin=[math.radians(22), math.radians(-159),0])
	f = ScenarioHelper.createPlatform(client, 20, [math.radians(22.1), math.radians(-159), 1000.0],
		'Python Platform', 0.0, 10.0, 'a-4m_skyhawk.3db') # slightly north
	if not f.isOk(2500):
		raise Exception("Failed createPlatform: " + f.responseDescription())
	platId = f.value().piData.platform.uniqueId

	# Create two ship platforms in the path of the projector
	ScenarioHelper.createPlatform(client, 21, [math.radians(22.1085), math.radians(-159.0005), 0.0],
		'Ship 1', 0.0, 10.0, 'uss_arleigh_burke_class_ddg')
	f = ScenarioHelper.createPlatform(client, 22, [math.radians(22.1085), math.radians(-158.99995), 0.0],
		'Ship 2', 0.0, 10.0, 'uss_arleigh_burke_class_ddg')
	if not f.isOk(2500):
		raise Exception("Failed createPlatform Ship 2: " + f.responseDescription())
	ship2Id = f.value().piData.platform.uniqueId

	# Add another data point to host platform that is looking downwards
	piData.addPlatformData(platId, 5.0, math.radians(22.1), math.radians(-159), 1000.0,
		[0.0, math.radians(-90), 0.0])

	# Update the time values in scenario
	piData.clockAutoConfigureBeginEnd()
	piData.clockSetTime(2.5)

	# Set a reasonable view
	piSimdis.tetherCamera("MainView", platId)
	piSimdis.setCameraRAE("MainView", 50.0, 0.0, math.radians(70.0))

	# Create a projector
	f = piData.createProjector(platId, "Projector", 100, 100)
	if not f.isOk(2500):
		raise Exception("Failed createProjector: " + f.responseDescription())
	projId = f.value();
	piSimdis.setPreference(projId, "projectorAlpha", "0.6")

	# Projector should draw on top of one ship
	f = piSimdis.setEntityAcceptsProjector(ship2Id, projId, True)
	if not f.isOk(2500):
		raise Exception("Failed setEntityAcceptsProjector: " + f.responseDescription())

	f = piData.addProjectorFov(projId, 0.0, 10.0)
	if not f.isOk(2500):
		raise Exception("Failed addProjectorFov: " + f.responseDescription())

	f = piData.setProjectorRasterFilename(projId, "explosion_0.png")
	if not f.isOk(2500):
		raise Exception("Failed setProjectorRasterFilename: " + f.responseDescription())

	f = piData.setStatus(projId, 0.1, True)
	if not f.isOk(2500):
		raise Exception("Failed setStatus: " + f.responseDescription())
	print("Check SIMDIS; you should see the explosion image.  Waiting 3 seconds...")
	time.sleep(3.0)

	# Create a texture
	f = piSimdis.addTextureStream(crossPng, 0, "Projector Image")
	if not f.isOk(2500):
		raise Exception("Failed addTextureStream: " + f.responseDescription())
	textureId = f.value()

	# Assign the texture to the projector
	f = piData.setProjectorRasterID(projId, textureId)
	if not f.isOk(2500):
		raise Exception("Failed setPlatformTextureIcon: " + f.responseDescription())
	print("Check SIMDIS; you should see a + icon on the projector.  Waiting 3 seconds...")
	time.sleep(3.0)

	# Turn off the overlay of the one icon
	piSimdis.setEntityAcceptsProjector(ship2Id, projId, False)
	print("Projector no longer overlays the platform.")

	# Attempt to update the PNG icon
	f = piSimdis.updateTextureStream(textureId, xPng, 0, 0)
	if not f.isOk(2500):
		raise Exception("Failed updateTextureStream: " + f.responseDescription())
	print("Check SIMDIS; you should see an X icon on the projector.  Waiting 3 seconds...")
	time.sleep(3.0)

	# Update the texture with a dark grey band 3x2 along the top-right
	greySquare = bytes(bytearray.fromhex('404040404040'))
	GL_TEXTURE_2D = 0x0DE1
	GL_LUMINANCE = 0x1909
	GL_UNSIGNED_BYTE = 0x1401
	f = piSimdis.updateTexture(textureId, GL_TEXTURE_2D, 0, 1, 2, 3, 2, GL_LUMINANCE, GL_UNSIGNED_BYTE, greySquare)
	if not f.isOk(2500):
		raise Exception("Failed updateTexture: " + f.responseDescription())
	print("Check SIMDIS; the X should have a grey panel on top-right corner.")
	time.sleep(3.0)

	# Replace the texture with a square, 4x4
	largeSquare = bytes(bytearray.fromhex('2020202020ffff2020ffff2020202020'))
	f = piSimdis.updateTexture(textureId, GL_TEXTURE_2D, 0,  0,0, 4,4, GL_LUMINANCE, GL_UNSIGNED_BYTE, largeSquare)
	if not f.isOk(2500):
		raise Exception("Failed updateTexture: " + f.responseDescription())
	print("Check SIMDIS; the icon should be a square now, white in middle and dark grey on outside")

	# Clean up the texture memory
	f = piSimdis.removeTexture(textureId)
	if not f.isOk(2500):
		raise Exception("Failed removeTexture: " + f.responseDescription())

	# Turn off the overlay of the one icon
	f = piSimdis.setEntityAcceptsProjector(ship2Id, projId, True)
	if not f.isOk(2500):
		raise Exception("Failed setEntityAcceptsProjector: " + f.responseDescription())
	print("Check SIMDIS; projector overlays a platform again.")
	time.sleep(3.0)

	# Create a beam holder platform that will also accept the projector
	f = ScenarioHelper.createPlatform(client, 30, [math.radians(22.1085), math.radians(-158.98995), 50.0],
		'BeamHost', 0.0, 10.0, 'a-4m_skyhawk.3db', ori=[math.radians(-90), 0, 0]) # slightly north
	if not f.isOk(2500):
		raise Exception("Failed createPlatform: " + f.responseDescription())
	beamHostId = f.value().piData.platform.uniqueId

	# Create the actual beam, pointing west
	f = piData.createBeam(beamHostId, "Beam 1", 9, math.radians(2.0), math.radians(2.0), 100)
	if not f.isOk(2500):
		raise Exception("Failed to create beam")
	beamId = f.value();
	piData.setColor(beamId, 0xffffffa0, 0.0)
	piData.setStatus(beamId, 0.0, True)
	piData.addBeamData(beamId, 0.0, math.radians(-90), 0, 10000)

	# Make sure we can overlay a beam
	piSimdis.setEntityAcceptsProjector(beamId, projId, True)
	print("Check SIMDIS; beam should be accepting the projector (not white in spots)")
	time.sleep(3.0)
	piSimdis.setEntityAcceptsProjector(beamId, projId, False)
	print("Check SIMDIS; beam should no longer accept projector")
	time.sleep(3.0)
	# End with the beam accepting the projector
	piSimdis.setEntityAcceptsProjector(beamId, projId, True)
	print("Beam should now be accepting projector again.")

finally:
	client.close()
