import io
from remoteapi import *
import PISIMDISFile_pb2

"""
TestSerialization.py

Unit test for the Remote API using Python that validates the serialization
and de-serialization of SIMDIS scenario objects.

"""

client = remoteapi.DiscoveryClient().createClient()

if client is None:
	raise Exception("Failed to connect.")

pi = client.pluginApi()
piData = pi.data()
piSimdis = pi.simdis()

pi.config().setResponsesEnabled(True)

# If a scenario already exists, then we'll get initialization errors in SIMDIS if we try to initialize
f = piData.isScenarioInitialized()
if not f.isOk(1000):
	raise Exception("Failed isScenarioInitialized")

if f.value():
	f = piData.deleteScenario()
	if not f.isOk(5000):
		raise Exception("Failed to delete scenario")

f = piData.initializeScenario("Python Test", 0, 0, 0)
if not f.isOk(5000):
	raise Exception("Failed to initialize scenario")

###########
# VIEW FILE

# De-serialize a view into SIMDIS
format = PISIMDISFile_pb2.FILE_FORMAT_VIEW
viewString = bytes(b"""
<Simdis10ViewFile>
 <Version>1.0</Version>
 <MainView>
  <FOV>60</FOV>
  <EyePosition>
   <Longitude>0</Longitude>
   <Latitude>0</Latitude>
   <Altitude>-9.3132257e-10</Altitude>
   <Azimuth>0</Azimuth>
   <Elevation>-89.8</Elevation>
   <Range>6500000</Range>
   <Overhead>False</Overhead>
   <Cockpit>False</Cockpit>
   <ReplaceCentered>False</ReplaceCentered>
   <TetherMode>Center</TetherMode>
   <LockHeading>False</LockHeading>
   <LockPitch>False</LockPitch>
  </EyePosition>
 </MainView>
</Simdis10ViewFile>
""")
option = PISIMDISFile_pb2.KeyValuePair()
option.key = "Append"
option.value = "True"
f = piSimdis.deserializeStream(format, viewString, [option])
if not f.isOk(60000):
	raise Exception("Failed to de-serialize view stream")

# Serialize the view that we loaded earlier
f = piSimdis.serializeToStream(format, [option])
print(f.value()) # view the data in the console to confirm that it is correct
if not f.isOk(60000):
	raise Exception("Failed to serialize the view that was loaded")

###########
# RULE FILE

# De-serialize rules into SIMDIS
format = PISIMDISFile_pb2.FILE_FORMAT_RULES
rulesString = bytes(b"""
Version 2
ruleName=GatesDrawMode ruleValue=solid  nameExpression=".*"  categoryFilters=" "
ruleName=GatesDrawCross ruleValue=yes  nameExpression=".*"  categoryFilters=" "
""")
f = piSimdis.deserializeStream(format, rulesString, [option])
if not f.isOk(60000):
	raise Exception("Failed to de-serialize rules stream")

# Serialize the rules that we loaded earlier
f = piSimdis.serializeToStream(format, [option])
print(f.value()) # view the data in the console to confirm that it is correct
if not f.isOk(60000):
	raise Exception("Failed to serialize the rules that were loaded")


############
# EARTH FILE
option = PISIMDISFile_pb2.KeyValuePair()
option.key = "RelativeTo"
option.value = "c:/people/feds/Distribution/"
f = piSimdis.serializeToStream(PISIMDISFile_pb2.FILE_FORMAT_EARTH, [option])
if not f.isOk(60000):
	raise Exception("Failed to serialize earth file")
print(f.value())
readyMapEarth = b"""
<map>
    <image name="readymap_imagery" driver="tms">
        <url>http://readymap.org/readymap/tiles/1.0.0/7/</url>
    </image>
    <elevation name="readymap_elevation" driver="tms">
        <url>http://readymap.org/readymap/tiles/1.0.0/116/</url>
    </elevation>
</map>
"""
option = PISIMDISFile_pb2.KeyValuePair()
option.key = "RelativeTo"
option.value = "c:/people/feds/Distribution/SIMDIS/reader.earth"


f = piSimdis.deserializeStream(PISIMDISFile_pb2.FILE_FORMAT_EARTH, readyMapEarth, [option])
if not f.isOk(60000):
	raise Exception("Failed to deserialize the ReadyMap Earth")

client.close()
print("Test successful")
